#!/usr/local/bin/perl
use strict;
use warnings;

use feature qw(state);
use ReadOscars qw(read_oscars);

my %ref_to_year;
my ( $oscar_ref, $movie_ref ) = read_oscars( 'oscars.txt' );
for my $year ( keys %$oscar_ref )
{
  for my $category ( keys %{ $oscar_ref->{$year} } )
  {
    my $ref = $oscar_ref->{$year}{$category};
    $ref_to_year{ $ref } = $year;    # Keys are stringified
  }
}

for my $movie ( keys %$movie_ref )
{
  for my $category ( keys %{ $movie_ref->{$movie} } )
  {
    my $ref = $movie_ref->{$movie}{$category};
    my $year = $ref_to_year{$ref} or die $movie;

    my $movie_id = insert_movie( $movie );
    for my $name ( @$ref )
    {
      my $person_id = insert_person( $name );
      insert_oscar( $year, $category, $person_id, $movie_id );
    }
  }
}

{
  my %movie_id;

  sub insert_movie
  {
    my $title = shift;

    $movie_id{$title} and return $movie_id{$title};
    state $creative_id = 0;
    state $movie_id = 0;
    $_++ for ( $creative_id, $movie_id );
    print <<"EOSQL";
  INSERT INTO creative_work ( id, title ) VALUES ( $creative_id, '\Q$title\E' );
  INSERT INTO movie ( id, creative_work_id  ) VALUES ( $movie_id, $creative_id );
EOSQL
    return $movie_id{$title} = $movie_id;
  }
}

sub insert_person
{
  my $name = shift;

  state $person_id = 0;
  $person_id++;
  print <<"EOSQL";
  INSERT INTO person ( id, name ) VALUES ( $person_id, '\Q$name\E' );
EOSQL
  return $person_id;
}

sub insert_oscar
{
  my ($year, $category, $person_id, $movie_id) = @_;

  state $oscar_id = 0;
  $oscar_id++;
  print <<"EOSQL";
  INSERT INTO oscar ( id, year, category, for_movie_id ) VALUES ( $oscar_id, $year, '$category', $movie_id );
EOSQL
  print <<"EOSQL";
  INSERT INTO winner_oscar_assoc ( oscar_id, winner_person_id ) VALUES ( $oscar_id, $person_id );
EOSQL
  print <<"EOSQL";
  INSERT INTO actor_movie_assoc ( movie_id, person_id ) VALUES ( $movie_id, $person_id )
EOSQL
}
